@@ -2,6 +2,7 @@ package kong
 
 import (
 	"fmt"
+	"os"
 	"reflect"
 	"sort"
 	"strconv"
@@ -43,6 +44,27 @@ func (p *Path) Node() *Node {
 	return nil
 }
 
+// Visitable returns the Visitable for this path element.
+func (p *Path) Visitable() Visitable {
+	switch {
+	case p.App != nil:
+		return p.App
+
+	case p.Argument != nil:
+		return p.Argument
+
+	case p.Command != nil:
+		return p.Command
+
+	case p.Flag != nil:
+		return p.Flag
+
+	case p.Positional != nil:
+		return p.Positional
+	}
+	return nil
+}
+
 // Context contains the current parse context.
 type Context struct {
 	*Kong
@@ -91,7 +113,8 @@ func (c *Context) Bind(args ...interface{}) {
 //
 //    BindTo(impl, (*MyInterface)(nil))
 func (c *Context) BindTo(impl, iface interface{}) {
-	c.bindings[reflect.TypeOf(iface).Elem()] = reflect.ValueOf(impl)
+	valueOf := reflect.ValueOf(impl)
+	c.bindings[reflect.TypeOf(iface).Elem()] = func() (reflect.Value, error) { return valueOf, nil }
 }
 
 // Value returns the value for a particular path element.
@@ -132,11 +155,21 @@ func (c *Context) Empty() bool {
 }
 
 // Validate the current context.
-func (c *Context) Validate() error {
+func (c *Context) Validate() error { // nolint: gocyclo
 	err := Visit(c.Model, func(node Visitable, next Next) error {
-		if value, ok := node.(*Value); ok {
-			if value.Enum != "" {
-				if err := checkEnum(value, value.Target); err != nil {
+		switch node := node.(type) {
+		case *Value:
+			_, ok := os.LookupEnv(node.Tag.Env)
+			if node.Enum != "" && (!node.Required || node.Default != "" || (node.Tag.Env != "" && ok)) {
+				if err := checkEnum(node, node.Target); err != nil {
+					return err
+				}
+			}
+
+		case *Flag:
+			_, ok := os.LookupEnv(node.Tag.Env)
+			if node.Enum != "" && (!node.Required || node.Default != "" || (node.Tag.Env != "" && ok)) {
+				if err := checkEnum(node.Value, node.Target); err != nil {
 					return err
 				}
 			}
@@ -146,12 +179,54 @@ func (c *Context) Validate() error {
 	if err != nil {
 		return err
 	}
+	for _, el := range c.Path {
+		var (
+			value reflect.Value
+			desc  string
+		)
+		switch node := el.Visitable().(type) {
+		case *Value:
+			value = node.Target
+			desc = node.ShortSummary()
+
+		case *Flag:
+			value = node.Target
+			desc = node.ShortSummary()
+
+		case *Application:
+			value = node.Target
+			desc = node.Name
+
+		case *Node:
+			value = node.Target
+			desc = node.Path()
+		}
+		if validate := isValidatable(value); validate != nil {
+			err := validate.Validate()
+			if err != nil {
+				return errors.Wrap(err, desc)
+			}
+		}
+	}
 	for _, resolver := range c.combineResolvers() {
 		if err := resolver.Validate(c.Model); err != nil {
 			return err
 		}
 	}
 	for _, path := range c.Path {
+		var value *Value
+		switch {
+		case path.Flag != nil:
+			value = path.Flag.Value
+
+		case path.Positional != nil:
+			value = path.Positional
+		}
+		if value != nil && value.Tag.Enum != "" {
+			if err := checkEnum(value, value.Target); err != nil {
+				return err
+			}
+		}
 		if err := checkMissingFlags(path.Flags); err != nil {
 			return err
 		}
@@ -249,6 +324,21 @@ func (c *Context) Reset() error {
 	})
 }
 
+func (c *Context) endParsing() {
+	args := []string{}
+	for {
+		token := c.scan.Pop()
+		if token.Type == EOLToken {
+			break
+		}
+		args = append(args, token.String())
+	}
+	// Note: tokens must be pushed in reverse order.
+	for i := range args {
+		c.scan.PushTyped(args[len(args)-1-i], PositionalArgumentToken)
+	}
+}
+
 func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 	positional := 0
 
@@ -265,21 +355,16 @@ func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 			case string:
 
 				switch {
+				case v == "-":
+					fallthrough
+				default: // nolint
+					c.scan.Pop()
+					c.scan.PushTyped(token.Value, PositionalArgumentToken)
+
 				// Indicates end of parsing. All remaining arguments are treated as positional arguments only.
 				case v == "--":
 					c.scan.Pop()
-					args := []string{}
-					for {
-						token = c.scan.Pop()
-						if token.Type == EOLToken {
-							break
-						}
-						args = append(args, token.String())
-					}
-					// Note: tokens must be pushed in reverse order.
-					for i := range args {
-						c.scan.PushTyped(args[len(args)-1-i], PositionalArgumentToken)
-					}
+					c.endParsing()
 
 				// Long flag.
 				case strings.HasPrefix(v, "--"):
@@ -299,10 +384,6 @@ func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 						c.scan.PushTyped(tail, ShortFlagTailToken)
 					}
 					c.scan.PushTyped(v[1:2], ShortFlagToken)
-
-				default:
-					c.scan.Pop()
-					c.scan.PushTyped(token.Value, PositionalArgumentToken)
 				}
 			default:
 				c.scan.Pop()
@@ -336,6 +417,11 @@ func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 			// Ensure we've consumed all positional arguments.
 			if positional < len(node.Positional) {
 				arg := node.Positional[positional]
+
+				if arg.Passthrough {
+					c.endParsing()
+				}
+
 				err := arg.Parse(c.scan, c.getValue(arg))
 				if err != nil {
 					return err
@@ -348,6 +434,24 @@ func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 				break
 			}
 
+			// Assign token value to a branch name if tagged as an alias
+			// An alias will be ignored in the case of an existing command
+			cmds := make(map[string]bool)
+			for _, branch := range node.Children {
+				if branch.Type == CommandNode {
+					cmds[branch.Name] = true
+				}
+			}
+			for _, branch := range node.Children {
+				for _, a := range branch.Aliases {
+					_, ok := cmds[a]
+					if token.Value == a && !ok {
+						token.Value = branch.Name
+						break
+					}
+				}
+			}
+
 			// After positional arguments have been consumed, check commands next...
 			for _, branch := range node.Children {
 				if branch.Type == CommandNode && !branch.Hidden {
@@ -384,6 +488,33 @@ func (c *Context) trace(node *Node) (err error) { // nolint: gocyclo
 			return fmt.Errorf("unexpected token %s", token)
 		}
 	}
+	return c.maybeSelectDefault(flags, node)
+}
+
+// End of the line, check for a default command, but only if we're not displaying help,
+// otherwise we'd only ever display the help for the default command.
+func (c *Context) maybeSelectDefault(flags []*Flag, node *Node) error {
+	for _, flag := range flags {
+		if flag.Name == "help" && flag.Set {
+			return nil
+		}
+	}
+	var defaultNode *Path
+	for _, child := range node.Children {
+		if child.Type == CommandNode && child.Tag.Default != "" {
+			if defaultNode != nil {
+				return fmt.Errorf("can't have more than one default command under %s", node.Summary())
+			}
+			defaultNode = &Path{
+				Parent:  child,
+				Command: child,
+				Flags:   child.Flags,
+			}
+		}
+	}
+	if defaultNode != nil {
+		c.Path = append(c.Path, defaultNode)
+	}
 	return nil
 }
 
@@ -401,26 +532,34 @@ func (c *Context) Resolve() error {
 			if _, ok := c.values[flag.Value]; ok {
 				continue
 			}
+
+			// Pick the last resolved value.
+			var selected interface{}
 			for _, resolver := range resolvers {
 				s, err := resolver.Resolve(c, path, flag)
 				if err != nil {
-					return err
+					return errors.Wrap(err, flag.ShortSummary())
 				}
 				if s == nil {
 					continue
 				}
-
-				scan := Scan().PushTyped(s, FlagValueToken)
-				delete(c.values, flag.Value)
-				err = flag.Parse(scan, c.getValue(flag.Value))
-				if err != nil {
-					return err
-				}
-				inserted = append(inserted, &Path{
-					Flag:     flag,
-					Resolved: true,
-				})
+				selected = s
 			}
+
+			if selected == nil {
+				continue
+			}
+
+			scan := Scan().PushTyped(selected, FlagValueToken)
+			delete(c.values, flag.Value)
+			err := flag.Parse(scan, c.getValue(flag.Value))
+			if err != nil {
+				return err
+			}
+			inserted = append(inserted, &Path{
+				Flag:     flag,
+				Resolved: true,
+			})
 		}
 	}
 	c.Path = append(inserted, c.Path...)
@@ -439,6 +578,14 @@ func (c *Context) getValue(value *Value) reflect.Value {
 	v, ok := c.values[value]
 	if !ok {
 		v = reflect.New(value.Target.Type()).Elem()
+		switch v.Kind() {
+		case reflect.Ptr:
+			v.Set(reflect.New(v.Type().Elem()))
+		case reflect.Slice:
+			v.Set(reflect.MakeSlice(v.Type(), 0, 0))
+		case reflect.Map:
+			v.Set(reflect.MakeMap(v.Type()))
+		}
 		c.values[value] = v
 	}
 	return v
@@ -501,15 +648,19 @@ func (c *Context) parseFlag(flags []*Flag, match string) (err error) {
 	for _, flag := range flags {
 		long := "--" + flag.Name
 		short := "-" + string(flag.Short)
+		neg := "--no-" + flag.Name
 		candidates = append(candidates, long)
 		if flag.Short != 0 {
 			candidates = append(candidates, short)
 		}
-		if short != match && long != match {
+		if short != match && long != match && !(match == neg && flag.Tag.Negatable) {
 			continue
 		}
 		// Found a matching flag.
 		c.scan.Pop()
+		if match == neg && flag.Tag.Negatable {
+			flag.Negated = true
+		}
 		err := flag.Parse(c.scan, c.getValue(flag.Value))
 		if err != nil {
 			if e, ok := errors.Cause(err).(*expectedError); ok && e.token.InferredType().IsAny(FlagToken, ShortFlagToken) {
@@ -517,22 +668,24 @@ func (c *Context) parseFlag(flags []*Flag, match string) (err error) {
 			}
 			return err
 		}
+		if flag.Negated {
+			value := c.getValue(flag.Value)
+			value.SetBool(!value.Bool())
+			flag.Value.Apply(value)
+		}
 		c.Path = append(c.Path, &Path{Flag: flag})
 		return nil
 	}
 	return findPotentialCandidates(match, candidates, "unknown flag %s", match)
 }
 
-// Run executes the Run() method on the selected command, which must exist.
+// RunNode calls the Run() method on an arbitrary node.
+//
+// This is useful in conjunction with Visit(), for dynamically running commands.
 //
 // Any passed values will be bindable to arguments of the target Run() method. Additionally,
 // all parent nodes in the command structure will be bound.
-func (c *Context) Run(binds ...interface{}) (err error) {
-	defer catch(&err)
-	node := c.Selected()
-	if node == nil {
-		return fmt.Errorf("no command selected")
-	}
+func (c *Context) RunNode(node *Node, binds ...interface{}) (err error) {
 	type targetMethod struct {
 		node   *Node
 		method reflect.Value
@@ -566,14 +719,26 @@ func (c *Context) Run(binds ...interface{}) (err error) {
 	return nil
 }
 
+// Run executes the Run() method on the selected command, which must exist.
+//
+// Any passed values will be bindable to arguments of the target Run() method. Additionally,
+// all parent nodes in the command structure will be bound.
+func (c *Context) Run(binds ...interface{}) (err error) {
+	defer catch(&err)
+	node := c.Selected()
+	if node == nil {
+		return fmt.Errorf("no command selected")
+	}
+	return c.RunNode(node, binds...)
+}
+
 // PrintUsage to Kong's stdout.
 //
 // If summary is true, a summarised version of the help will be output.
 func (c *Context) PrintUsage(summary bool) error {
 	options := c.helpOptions
 	options.Summary = summary
-	_ = c.help(options, c)
-	return nil
+	return c.help(options, c)
 }
 
 func checkMissingFlags(flags []*Flag) error {
@@ -604,6 +769,7 @@ func checkMissingChildren(node *Node) error {
 		missing = append(missing, strconv.Quote(strings.Join(missingArgs, " ")))
 	}
 
+	haveDefault := 0
 	for _, child := range node.Children {
 		if child.Hidden {
 			continue
@@ -614,10 +780,19 @@ func checkMissingChildren(node *Node) error {
 			}
 			missing = append(missing, strconv.Quote(child.Summary()))
 		} else {
+			if child.Tag.Default != "" {
+				if len(child.Children) > 0 {
+					return fmt.Errorf("default command %s must not have subcommands or arguments", child.Summary())
+				}
+				haveDefault++
+			}
 			missing = append(missing, strconv.Quote(child.Name))
 		}
 	}
-	if len(missing) == 0 {
+	if haveDefault > 1 {
+		return fmt.Errorf("more than one default command found under %s", node.Summary())
+	}
+	if len(missing) == 0 || haveDefault > 0 {
 		return nil
 	}
 
@@ -714,3 +889,18 @@ func findPotentialCandidates(needle string, haystack []string, format string, ar
 	}
 	return fmt.Errorf("%s", prefix)
 }
+
+type validatable interface{ Validate() error }
+
+func isValidatable(v reflect.Value) validatable {
+	if !v.IsValid() || (v.Kind() == reflect.Ptr || v.Kind() == reflect.Slice || v.Kind() == reflect.Map) && v.IsNil() {
+		return nil
+	}
+	if validate, ok := v.Interface().(validatable); ok {
+		return validate
+	}
+	if v.CanAddr() {
+		return isValidatable(v.Addr())
+	}
+	return nil
+}
