@@ -31,6 +31,13 @@ func Must(ast interface{}, options ...Option) *Kong {
 	return k
 }
 
+type usageOnError int
+
+const (
+	shortUsage usageOnError = iota + 1
+	fullUsage
+)
+
 // Kong is the main parser type.
 type Kong struct {
 	// Grammar model.
@@ -48,15 +55,18 @@ type Kong struct {
 	registry  *Registry
 
 	noDefaultHelp bool
-	usageOnError  bool
+	usageOnError  usageOnError
 	help          HelpPrinter
+	shortHelp     HelpPrinter
 	helpFormatter HelpValueFormatter
 	helpOptions   HelpOptions
 	helpFlag      *Flag
+	groups        []Group
 	vars          Vars
 
 	// Set temporarily by Options. These are applied after build().
 	postBuildOptions []Option
+	dynamicCommands  []*dynamicCommand
 }
 
 // New creates a new Kong parser on grammar.
@@ -85,6 +95,10 @@ func New(grammar interface{}, options ...Option) (*Kong, error) {
 		k.help = DefaultHelpPrinter
 	}
 
+	if k.shortHelp == nil {
+		k.shortHelp = DefaultShortHelpPrinter
+	}
+
 	model, err := build(k, grammar)
 	if err != nil {
 		return k, err
@@ -93,6 +107,20 @@ func New(grammar interface{}, options ...Option) (*Kong, error) {
 	k.Model = model
 	k.Model.HelpFlag = k.helpFlag
 
+	// Synthesise command nodes.
+	for _, dcmd := range k.dynamicCommands {
+		tag := newEmptyTag()
+		tag.Name = dcmd.name
+		tag.Help = dcmd.help
+		tag.Group = dcmd.group
+		tag.Cmd = true
+		v := reflect.Indirect(reflect.ValueOf(dcmd.cmd))
+		buildChild(k, k.Model.Node, CommandNode, reflect.Value{}, reflect.StructField{
+			Name: dcmd.name,
+			Type: v.Type(),
+		}, v, tag, dcmd.name, map[string]bool{})
+	}
+
 	for _, option := range k.postBuildOptions {
 		if err = option.Apply(k); err != nil {
 			return nil, err
@@ -128,7 +156,7 @@ func (k *Kong) interpolate(node *Node) (err error) {
 		switch node := node.(type) {
 		case *Node:
 			vars := stack.push(node.Vars())
-			node.Help, err = interpolate(node.Help, vars)
+			node.Help, err = interpolate(node.Help, vars, nil)
 			if err != nil {
 				return fmt.Errorf("help for %s: %s", node.Path(), err)
 			}
@@ -144,18 +172,20 @@ func (k *Kong) interpolate(node *Node) (err error) {
 }
 
 func (k *Kong) interpolateValue(value *Value, vars Vars) (err error) {
-	vars = vars.CloneWith(value.Tag.Vars)
-	if value.Default, err = interpolate(value.Default, vars); err != nil {
+	if len(value.Tag.Vars) > 0 {
+		vars = vars.CloneWith(value.Tag.Vars)
+	}
+	if value.Default, err = interpolate(value.Default, vars, nil); err != nil {
 		return fmt.Errorf("default value for %s: %s", value.Summary(), err)
 	}
-	if value.Enum, err = interpolate(value.Enum, vars); err != nil {
+	if value.Enum, err = interpolate(value.Enum, vars, nil); err != nil {
 		return fmt.Errorf("enum value for %s: %s", value.Summary(), err)
 	}
-	vars = vars.CloneWith(map[string]string{
+	value.Help, err = interpolate(value.Help, vars, map[string]string{
 		"default": value.Default,
 		"enum":    value.Enum,
 	})
-	if value.Help, err = interpolate(value.Help, vars); err != nil {
+	if err != nil {
 		return fmt.Errorf("help for %s: %s", value.Summary(), err)
 	}
 	return nil
@@ -169,6 +199,7 @@ func (k *Kong) extraFlags() []*Flag {
 	var helpTarget helpValue
 	value := reflect.ValueOf(&helpTarget).Elem()
 	helpFlag := &Flag{
+		Short: 'h',
 		Value: &Value{
 			Name:         "help",
 			Help:         "Show context-sensitive help.",
@@ -327,13 +358,17 @@ func (k *Kong) FatalIfErrorf(err error, args ...interface{}) {
 		msg = fmt.Sprintf(args[0].(string), args[1:]...) + ": " + err.Error()
 	}
 	// Maybe display usage information.
-	if err, ok := err.(*ParseError); ok && k.usageOnError {
-		options := k.helpOptions
-		_ = k.help(options, err.Context)
-		fmt.Fprintln(k.Stdout)
+	if err, ok := err.(*ParseError); ok {
+		switch k.usageOnError {
+		case fullUsage:
+			_ = k.help(k.helpOptions, err.Context)
+			fmt.Fprintln(k.Stdout)
+		case shortUsage:
+			_ = k.shortHelp(k.helpOptions, err.Context)
+			fmt.Fprintln(k.Stdout)
+		}
 	}
-	k.Errorf("%s", msg)
-	k.Exit(1)
+	k.Fatalf("%s", msg)
 }
 
 // LoadConfig from path using the loader configured via Configuration(loader).
@@ -342,7 +377,7 @@ func (k *Kong) FatalIfErrorf(err error, args ...interface{}) {
 func (k *Kong) LoadConfig(path string) (Resolver, error) {
 	var err error
 	path = ExpandPath(path)
-	path, err = interpolate(path, k.vars)
+	path, err = interpolate(path, k.vars, nil)
 	if err != nil {
 		return nil, err
 	}
