@@ -6,6 +6,7 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/pkg/errors"
 	"github.com/stretchr/testify/require"
 
 	"github.com/alecthomas/kong"
@@ -355,6 +356,94 @@ func TestTraceErrorPartiallySucceeds(t *testing.T) {
 	require.Equal(t, "one", ctx.Command())
 }
 
+type commandWithNegatableFlag struct {
+	Flag bool `kong:"default='true',negatable"`
+	ran  bool
+}
+
+func (c *commandWithNegatableFlag) Run() error {
+	c.ran = true
+	return nil
+}
+
+func TestNegatableFlag(t *testing.T) {
+	tests := []struct {
+		name     string
+		args     []string
+		expected bool
+	}{
+		{
+			name:     "no flag",
+			args:     []string{"cmd"},
+			expected: true,
+		},
+		{
+			name:     "boolean flag",
+			args:     []string{"cmd", "--flag"},
+			expected: true,
+		},
+		{
+			name:     "inverted boolean flag",
+			args:     []string{"cmd", "--flag=false"},
+			expected: false,
+		},
+		{
+			name:     "negated boolean flag",
+			args:     []string{"cmd", "--no-flag"},
+			expected: false,
+		},
+		{
+			name:     "inverted negated boolean flag",
+			args:     []string{"cmd", "--no-flag=false"},
+			expected: true,
+		},
+	}
+	for _, tt := range tests {
+		tt := tt
+		t.Run(tt.name, func(t *testing.T) {
+			var cli struct {
+				Cmd commandWithNegatableFlag `kong:"cmd"`
+			}
+
+			p := mustNew(t, &cli)
+			kctx, err := p.Parse(tt.args)
+			require.NoError(t, err)
+			require.Equal(t, tt.expected, cli.Cmd.Flag)
+
+			err = kctx.Run()
+			require.NoError(t, err)
+			require.Equal(t, tt.expected, cli.Cmd.Flag)
+			require.True(t, cli.Cmd.ran)
+		})
+	}
+}
+
+func TestExistingNoFlag(t *testing.T) {
+	var cli struct {
+		Cmd struct {
+			Flag   bool `kong:"default='true'"`
+			NoFlag string
+		} `kong:"cmd"`
+	}
+
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"cmd", "--no-flag=none"})
+	require.NoError(t, err)
+	require.Equal(t, true, cli.Cmd.Flag)
+	require.Equal(t, "none", cli.Cmd.NoFlag)
+}
+
+func TestInvalidNegatedNonBool(t *testing.T) {
+	var cli struct {
+		Cmd struct {
+			Flag string `kong:"negatable"`
+		} `kong:"cmd"`
+	}
+
+	_, err := kong.New(&cli)
+	require.Error(t, err)
+}
+
 type hookContext struct {
 	cmd    bool
 	values []string
@@ -743,7 +832,7 @@ func TestEnvarEnumValidated(t *testing.T) {
 	})
 	defer restore()
 	var cli struct {
-		Flag string `env:"FLAG" enum:"valid" default:"valid"`
+		Flag string `env:"FLAG" required:"" enum:"valid"`
 	}
 	p := mustNew(t, &cli)
 	_, err := p.Parse(nil)
@@ -791,3 +880,188 @@ func TestEnumSequence(t *testing.T) {
 	require.NoError(t, err)
 	require.Equal(t, []string{"a"}, cli.State)
 }
+
+func TestIssue40EnumAcrossCommands(t *testing.T) {
+	var cli struct {
+		One struct {
+			OneArg string `arg:"" required:""`
+		} `cmd:""`
+		Two struct {
+			TwoArg string `arg:"" enum:"a,b,c" required:"" env:"FOO"`
+		} `cmd:""`
+		Three struct {
+			ThreeArg string `arg:"" optional:"" default:"a" enum:"a,b,c"`
+		} `cmd:""`
+	}
+
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"one", "two"})
+	require.NoError(t, err)
+	_, err = p.Parse([]string{"two", "d"})
+	require.Error(t, err)
+	_, err = p.Parse([]string{"three", "d"})
+	require.Error(t, err)
+	_, err = p.Parse([]string{"three", "c"})
+	require.NoError(t, err)
+}
+
+func TestEnumArg(t *testing.T) {
+	var cli struct {
+		Nested struct {
+			One string `arg:"" enum:"a,b,c"`
+			Two string `arg:""`
+		} `cmd:""`
+	}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"nested", "a", "b"})
+	require.NoError(t, err)
+	require.Equal(t, "a", cli.Nested.One)
+	require.Equal(t, "b", cli.Nested.Two)
+}
+
+func TestDefaultCommand(t *testing.T) {
+	var cli struct {
+		One struct{} `cmd:"" default:"1"`
+		Two struct{} `cmd:""`
+	}
+	p := mustNew(t, &cli)
+	ctx, err := p.Parse([]string{})
+	require.NoError(t, err)
+	require.Equal(t, "one", ctx.Command())
+}
+
+func TestMultipleDefaultCommands(t *testing.T) {
+	var cli struct {
+		One struct{} `cmd:"" default:"1"`
+		Two struct{} `cmd:"" default:"1"`
+	}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{})
+	require.EqualError(t, err, "can't have more than one default command under  <command>")
+}
+
+func TestLoneHpyhen(t *testing.T) {
+	var cli struct {
+		Flag string
+		Arg  string `arg:"" optional:""`
+	}
+	p := mustNew(t, &cli)
+
+	_, err := p.Parse([]string{"-"})
+	require.NoError(t, err)
+	require.Equal(t, "-", cli.Arg)
+
+	_, err = p.Parse([]string{"--flag", "-"})
+	require.NoError(t, err)
+	require.Equal(t, "-", cli.Flag)
+}
+
+func TestPlugins(t *testing.T) {
+	var pluginOne struct {
+		One string
+	}
+	var pluginTwo struct {
+		Two string
+	}
+	var cli struct {
+		Base string
+		kong.Plugins
+	}
+	cli.Plugins = kong.Plugins{&pluginOne, &pluginTwo}
+
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"--base=base", "--one=one", "--two=two"})
+	require.NoError(t, err)
+	require.Equal(t, "base", cli.Base)
+	require.Equal(t, "one", pluginOne.One)
+	require.Equal(t, "two", pluginTwo.Two)
+}
+
+type validateCmd struct{}
+
+func (v *validateCmd) Validate() error { return errors.New("cmd error") }
+
+type validateCli struct {
+	Cmd validateCmd `cmd:""`
+}
+
+func (v *validateCli) Validate() error { return errors.New("app error") }
+
+type validateFlag string
+
+func (v *validateFlag) Validate() error { return errors.New("flag error") }
+
+func TestValidateApp(t *testing.T) {
+	cli := validateCli{}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{})
+	require.EqualError(t, err, "test: app error")
+}
+
+func TestValidateCmd(t *testing.T) {
+	cli := struct {
+		Cmd validateCmd `cmd:""`
+	}{}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"cmd"})
+	require.EqualError(t, err, "cmd: cmd error")
+}
+
+func TestValidateFlag(t *testing.T) {
+	cli := struct {
+		Flag validateFlag
+	}{}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"--flag=one"})
+	require.EqualError(t, err, "--flag: flag error")
+}
+
+func TestValidateArg(t *testing.T) {
+	cli := struct {
+		Arg validateFlag `arg:""`
+	}{}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"one"})
+	require.EqualError(t, err, "<arg>: flag error")
+}
+
+func TestPointers(t *testing.T) {
+	cli := struct {
+		Mapped *mappedValue
+		JSON   *jsonUnmarshalerValue
+	}{}
+	p := mustNew(t, &cli)
+	_, err := p.Parse([]string{"--mapped=mapped", "--json=\"foo\""})
+	require.NoError(t, err)
+	require.NotNil(t, cli.Mapped)
+	require.Equal(t, "mapped", cli.Mapped.decoded)
+	require.NotNil(t, cli.JSON)
+	require.Equal(t, "FOO", string(*cli.JSON))
+}
+
+type dynamicCommand struct {
+	Flag string
+
+	ran bool
+}
+
+func (d *dynamicCommand) Run() error {
+	d.ran = true
+	return nil
+}
+
+func TestDynamicCommands(t *testing.T) {
+	cli := struct {
+		One struct{} `cmd:"one"`
+	}{}
+	two := &dynamicCommand{}
+	var twoi interface{} = &two
+	p := mustNew(t, &cli, kong.DynamicCommand("two", "", "", twoi))
+	kctx, err := p.Parse([]string{"two", "--flag=flag"})
+	require.NoError(t, err)
+	require.Equal(t, "flag", two.Flag)
+	require.False(t, two.ran)
+	err = kctx.Run()
+	require.NoError(t, err)
+	require.True(t, two.ran)
+}
