@@ -3,6 +3,7 @@ package kong
 import (
 	"encoding"
 	"encoding/json"
+	"fmt"
 	"io/ioutil"
 	"math/bits"
 	"net/url"
@@ -18,6 +19,7 @@ import (
 var (
 	mapperValueType       = reflect.TypeOf((*MapperValue)(nil)).Elem()
 	boolMapperType        = reflect.TypeOf((*BoolMapper)(nil)).Elem()
+	jsonUnmarshalerType   = reflect.TypeOf((*json.Unmarshaler)(nil)).Elem()
 	textUnmarshalerType   = reflect.TypeOf((*encoding.TextUnmarshaler)(nil)).Elem()
 	binaryUnmarshalerType = reflect.TypeOf((*encoding.BinaryUnmarshaler)(nil)).Elem()
 )
@@ -41,6 +43,7 @@ func (r *DecodeContext) WithScanner(scan *Scanner) *DecodeContext {
 }
 
 // MapperValue may be implemented by fields in order to provide custom mapping.
+// Mappers may additionally implement PlaceHolderProvider to provide custom placeholder text.
 type MapperValue interface {
 	Decode(ctx *DecodeContext) error
 }
@@ -82,17 +85,31 @@ func (m *binaryUnmarshalerAdapter) Decode(ctx *DecodeContext, target reflect.Val
 	if err != nil {
 		return err
 	}
-	if target.Type().Implements(textUnmarshalerType) {
+	if target.Type().Implements(binaryUnmarshalerType) {
 		return target.Interface().(encoding.BinaryUnmarshaler).UnmarshalBinary([]byte(value))
 	}
 	return target.Addr().Interface().(encoding.BinaryUnmarshaler).UnmarshalBinary([]byte(value))
 }
 
+type jsonUnmarshalerAdapter struct{}
+
+func (j *jsonUnmarshalerAdapter) Decode(ctx *DecodeContext, target reflect.Value) error {
+	var value string
+	err := ctx.Scan.PopValueInto("value", &value)
+	if err != nil {
+		return err
+	}
+	if target.Type().Implements(jsonUnmarshalerType) {
+		return target.Interface().(json.Unmarshaler).UnmarshalJSON([]byte(value))
+	}
+	return target.Addr().Interface().(json.Unmarshaler).UnmarshalJSON([]byte(value))
+}
+
 // A Mapper represents how a field is mapped from command-line values to Go.
 //
 // Mappers can be associated with concrete fields via pointer, reflect.Type, reflect.Kind, or via a "type" tag.
 //
-// Additionally, if a type implements the MappverValue interface, it will be used.
+// Additionally, if a type implements the MapperValue interface, it will be used.
 type Mapper interface {
 	// Decode ctx.Value with ctx.Scanner into target.
 	Decode(ctx *DecodeContext, target reflect.Value) error
@@ -176,10 +193,13 @@ func (r *Registry) ForType(typ reflect.Type) Mapper {
 	}
 	// Next try stdlib unmarshaler interfaces.
 	for _, impl := range []reflect.Type{typ, reflect.PtrTo(typ)} {
-		if impl.Implements(textUnmarshalerType) {
+		switch {
+		case impl.Implements(textUnmarshalerType):
 			return &textUnmarshalerAdapter{}
-		} else if impl.Implements(binaryUnmarshalerType) {
+		case impl.Implements(binaryUnmarshalerType):
 			return &binaryUnmarshalerAdapter{}
+		case impl.Implements(jsonUnmarshalerType):
+			return &jsonUnmarshalerAdapter{}
 		}
 	}
 	// Finally try registered kinds.
@@ -230,8 +250,8 @@ func (r *Registry) RegisterDefaults() *Registry {
 		RegisterKind(reflect.Int16, intDecoder(16)).
 		RegisterKind(reflect.Int32, intDecoder(32)).
 		RegisterKind(reflect.Int64, intDecoder(64)).
-		RegisterKind(reflect.Uint, uintDecoder(64)).
-		RegisterKind(reflect.Uint8, uintDecoder(bits.UintSize)).
+		RegisterKind(reflect.Uint, uintDecoder(bits.UintSize)).
+		RegisterKind(reflect.Uint8, uintDecoder(8)).
 		RegisterKind(reflect.Uint16, uintDecoder(16)).
 		RegisterKind(reflect.Uint32, uintDecoder(32)).
 		RegisterKind(reflect.Uint64, uintDecoder(64)).
@@ -247,6 +267,7 @@ func (r *Registry) RegisterDefaults() *Registry {
 		RegisterType(reflect.TypeOf(time.Time{}), timeDecoder()).
 		RegisterType(reflect.TypeOf(time.Duration(0)), durationDecoder()).
 		RegisterType(reflect.TypeOf(&url.URL{}), urlMapper()).
+		RegisterType(reflect.TypeOf(&os.File{}), fileMapper(r)).
 		RegisterName("path", pathMapper(r)).
 		RegisterName("existingfile", existingFileMapper(r)).
 		RegisterName("existingdir", existingDirMapper(r)).
@@ -325,23 +346,22 @@ func intDecoder(bits int) MapperFunc { // nolint: dupl
 		if err != nil {
 			return err
 		}
+		var sv string
 		switch v := t.Value.(type) {
 		case string:
-			n, err := strconv.ParseInt(v, 10, bits)
-			if err != nil {
-				return errors.Errorf("expected an int but got %q (%T)", t, t.Value)
-			}
-			target.SetInt(n)
+			sv = v
 
-		case float64:
-			target.SetInt(int64(v))
-
-		case int, int8, int16, int32, int64, uint, uint8, uint16, uint32, uint64:
-			target.Set(reflect.ValueOf(v))
+		case int, int8, int16, int32, int64, uint, uint8, uint16, uint32, uint64, float32, float64:
+			sv = fmt.Sprintf("%v", v)
 
 		default:
 			return errors.Errorf("expected an int but got %q (%T)", t, t.Value)
 		}
+		n, err := strconv.ParseInt(sv, 10, bits)
+		if err != nil {
+			return errors.Errorf("expected a valid %d bit int but got %q", bits, sv)
+		}
+		target.SetInt(n)
 		return nil
 	}
 }
@@ -352,23 +372,22 @@ func uintDecoder(bits int) MapperFunc { // nolint: dupl
 		if err != nil {
 			return err
 		}
+		var sv string
 		switch v := t.Value.(type) {
 		case string:
-			n, err := strconv.ParseUint(v, 10, bits)
-			if err != nil {
-				return errors.Errorf("expected a uint but got %q (%T)", t, t.Value)
-			}
-			target.SetUint(n)
+			sv = v
 
-		case float64:
-			target.SetUint(uint64(v))
-
-		case int, int8, int16, int32, int64, uint, uint8, uint16, uint32, uint64:
-			target.Set(reflect.ValueOf(v))
+		case int, int8, int16, int32, int64, uint, uint8, uint16, uint32, uint64, float32, float64:
+			sv = fmt.Sprintf("%v", v)
 
 		default:
 			return errors.Errorf("expected an int but got %q (%T)", t, t.Value)
 		}
+		n, err := strconv.ParseUint(sv, 10, bits)
+		if err != nil {
+			return errors.Errorf("expected a valid %d bit uint but got %q", bits, sv)
+		}
+		target.SetUint(n)
 		return nil
 	}
 }
@@ -536,12 +555,39 @@ func pathMapper(r *Registry) MapperFunc {
 		if err != nil {
 			return err
 		}
-		path = ExpandPath(path)
+		if path != "-" {
+			path = ExpandPath(path)
+		}
 		target.SetString(path)
 		return nil
 	}
 }
 
+func fileMapper(r *Registry) MapperFunc {
+	return func(ctx *DecodeContext, target reflect.Value) error {
+		if target.Kind() == reflect.Slice {
+			return sliceDecoder(r)(ctx, target)
+		}
+		var path string
+		err := ctx.Scan.PopValueInto("file", &path)
+		if err != nil {
+			return err
+		}
+		var file *os.File
+		if path == "-" {
+			file = os.Stdin
+		} else {
+			path = ExpandPath(path)
+			file, err = os.Open(path) // nolint: gosec
+			if err != nil {
+				return err
+			}
+		}
+		target.Set(reflect.ValueOf(file))
+		return nil
+	}
+}
+
 func existingFileMapper(r *Registry) MapperFunc {
 	return func(ctx *DecodeContext, target reflect.Value) error {
 		if target.Kind() == reflect.Slice {
@@ -555,13 +601,15 @@ func existingFileMapper(r *Registry) MapperFunc {
 		if err != nil {
 			return err
 		}
-		path = ExpandPath(path)
-		stat, err := os.Stat(path)
-		if err != nil {
-			return err
-		}
-		if stat.IsDir() {
-			return errors.Errorf("%q exists but is a directory", path)
+		if path != "-" {
+			path = ExpandPath(path)
+			stat, err := os.Stat(path)
+			if err != nil {
+				return err
+			}
+			if stat.IsDir() {
+				return errors.Errorf("%q exists but is a directory", path)
+			}
 		}
 		target.SetString(path)
 		return nil
@@ -596,6 +644,28 @@ func existingDirMapper(r *Registry) MapperFunc {
 
 func counterMapper() MapperFunc {
 	return func(ctx *DecodeContext, target reflect.Value) error {
+		if ctx.Scan.Peek().Type == FlagValueToken {
+			t, err := ctx.Scan.PopValue("counter")
+			if err != nil {
+				return err
+			}
+			switch v := t.Value.(type) {
+			case string:
+				n, err := strconv.ParseInt(v, 10, 64)
+				if err != nil {
+					return errors.Errorf("expected a counter but got %q (%T)", t, t.Value)
+				}
+				target.SetInt(n)
+
+			case int, int8, int16, int32, int64, uint, uint8, uint16, uint32, uint64:
+				target.Set(reflect.ValueOf(v))
+
+			default:
+				return errors.Errorf("expected a counter but got %q (%T)", t, t.Value)
+			}
+			return nil
+		}
+
 		switch target.Kind() {
 		case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
 			target.SetInt(target.Int() + 1)
@@ -635,6 +705,9 @@ func urlMapper() MapperFunc {
 //
 //     SplitEscaped(`hello\,there,bob`, ',') == []string{"hello,there", "bob"}
 func SplitEscaped(s string, sep rune) (out []string) {
+	if sep == -1 {
+		return []string{s}
+	}
 	escaped := false
 	token := ""
 	for _, ch := range s {
@@ -669,6 +742,33 @@ func JoinEscaped(s []string, sep rune) string {
 	return strings.Join(escaped, string(sep))
 }
 
+// NamedFileContentFlag is a flag value that loads a file's contents and filename into its value.
+type NamedFileContentFlag struct {
+	Filename string
+	Contents []byte
+}
+
+func (f *NamedFileContentFlag) Decode(ctx *DecodeContext) error { // nolint: golint
+	var filename string
+	err := ctx.Scan.PopValueInto("filename", &filename)
+	if err != nil {
+		return err
+	}
+	// This allows unsetting of file content flags.
+	if filename == "" {
+		*f = NamedFileContentFlag{}
+		return nil
+	}
+	filename = ExpandPath(filename)
+	data, err := ioutil.ReadFile(filename) // nolint: gosec
+	if err != nil {
+		return errors.Errorf("failed to open %q: %s", filename, err)
+	}
+	f.Contents = data
+	f.Filename = filename
+	return nil
+}
+
 // FileContentFlag is a flag value that loads a file's contents into its value.
 type FileContentFlag []byte
 
@@ -678,6 +778,11 @@ func (f *FileContentFlag) Decode(ctx *DecodeContext) error { // nolint: golint
 	if err != nil {
 		return err
 	}
+	// This allows unsetting of file content flags.
+	if filename == "" {
+		*f = nil
+		return nil
+	}
 	filename = ExpandPath(filename)
 	data, err := ioutil.ReadFile(filename) // nolint: gosec
 	if err != nil {
