@@ -2,6 +2,7 @@ package kong
 
 import (
 	"io"
+	"os"
 	"os/user"
 	"path/filepath"
 	"reflect"
@@ -35,7 +36,7 @@ func (v Vars) Apply(k *Kong) error {
 
 // CloneWith clones the current Vars and merges "vars" onto the clone.
 func (v Vars) CloneWith(vars Vars) Vars {
-	out := Vars{}
+	out := make(Vars, len(v)+len(vars))
 	for key, value := range v {
 		out[key] = value
 	}
@@ -53,6 +54,28 @@ func Exit(exit func(int)) Option {
 	})
 }
 
+type dynamicCommand struct {
+	name  string
+	help  string
+	group string
+	cmd   interface{}
+}
+
+// DynamicCommand registers a dynamically constructed command with the root of the CLI.
+//
+// This is useful for command-line structures that are extensible via user-provided plugins.
+func DynamicCommand(name, help, group string, cmd interface{}) Option {
+	return OptionFunc(func(k *Kong) error {
+		k.dynamicCommands = append(k.dynamicCommands, &dynamicCommand{
+			name:  name,
+			help:  help,
+			group: group,
+			cmd:   cmd,
+		})
+		return nil
+	})
+}
+
 // NoDefaultHelp disables the default help flags.
 func NoDefaultHelp() Option {
 	return OptionFunc(func(k *Kong) error {
@@ -61,24 +84,29 @@ func NoDefaultHelp() Option {
 	})
 }
 
+// PostBuild provides read/write access to kong.Kong after initial construction of the model is complete but before
+// parsing occurs.
+//
+// This is useful for, e.g., adding short options to flags, updating help, etc.
+func PostBuild(fn func(*Kong) error) Option {
+	return OptionFunc(func(k *Kong) error {
+		k.postBuildOptions = append(k.postBuildOptions, OptionFunc(fn))
+		return nil
+	})
+}
+
 // Name overrides the application name.
 func Name(name string) Option {
-	return OptionFunc(func(k *Kong) error {
-		k.postBuildOptions = append(k.postBuildOptions, OptionFunc(func(k *Kong) error {
-			k.Model.Name = name
-			return nil
-		}))
+	return PostBuild(func(k *Kong) error {
+		k.Model.Name = name
 		return nil
 	})
 }
 
 // Description sets the application description.
 func Description(description string) Option {
-	return OptionFunc(func(k *Kong) error {
-		k.postBuildOptions = append(k.postBuildOptions, OptionFunc(func(k *Kong) error {
-			k.Model.Help = description
-			return nil
-		}))
+	return PostBuild(func(k *Kong) error {
+		k.Model.Help = description
 		return nil
 	})
 }
@@ -186,6 +214,17 @@ func Help(help HelpPrinter) Option {
 	})
 }
 
+// ShortHelp configures the short usage message.
+//
+// It should be used together with kong.ShortUsageOnError() to display a
+// custom short usage message on errors.
+func ShortHelp(shortHelp HelpPrinter) Option {
+	return OptionFunc(func(k *Kong) error {
+		k.shortHelp = shortHelp
+		return nil
+	})
+}
+
 // HelpFormatter configures how the help text is formatted.
 func HelpFormatter(helpFormatter HelpValueFormatter) Option {
 	return OptionFunc(func(k *Kong) error {
@@ -202,10 +241,60 @@ func ConfigureHelp(options HelpOptions) Option {
 	})
 }
 
+// Groups associates `group` field tags with group metadata.
+//
+// This option is used to simplify Kong tags while providing
+// rich group information such as title and optional description.
+//
+// Each key in the "groups" map corresponds to the value of a
+// `group` Kong tag, while the first line of the value will be
+// the title, and subsequent lines if any will be the description of
+// the group.
+//
+// See also ExplicitGroups for a more structured alternative.
+type Groups map[string]string
+
+func (g Groups) Apply(k *Kong) error { // nolint: golint
+	for key, info := range g {
+		lines := strings.Split(info, "\n")
+		title := strings.TrimSpace(lines[0])
+		description := ""
+		if len(lines) > 1 {
+			description = strings.TrimSpace(strings.Join(lines[1:], "\n"))
+		}
+		k.groups = append(k.groups, Group{
+			Key:         key,
+			Title:       title,
+			Description: description,
+		})
+	}
+	return nil
+}
+
+// ExplicitGroups associates `group` field tags with their metadata.
+//
+// It can be used to provide a title or header to a command or flag group.
+func ExplicitGroups(groups []Group) Option {
+	return OptionFunc(func(k *Kong) error {
+		k.groups = groups
+		return nil
+	})
+}
+
 // UsageOnError configures Kong to display context-sensitive usage if FatalIfErrorf is called with an error.
 func UsageOnError() Option {
 	return OptionFunc(func(k *Kong) error {
-		k.usageOnError = true
+		k.usageOnError = fullUsage
+		return nil
+	})
+}
+
+// ShortUsageOnError configures Kong to display context-sensitive short
+// usage if FatalIfErrorf is called with an error. The default short
+// usage message can be overridden with kong.ShortHelp(...).
+func ShortUsageOnError() Option {
+	return OptionFunc(func(k *Kong) error {
+		k.usageOnError = shortUsage
 		return nil
 	})
 }
@@ -240,7 +329,20 @@ func Configuration(loader ConfigurationLoader, paths ...string) Option {
 	return OptionFunc(func(k *Kong) error {
 		k.loader = loader
 		for _, path := range paths {
-			resolver, _ := k.LoadConfig(path)
+			f, err := os.Open(ExpandPath(path))
+			if err != nil {
+				if os.IsNotExist(err) || os.IsPermission(err) {
+					continue
+				}
+
+				return err
+			}
+			f.Close()
+
+			resolver, err := k.LoadConfig(path)
+			if err != nil {
+				return errors.Wrap(err, path)
+			}
 			if resolver != nil {
 				k.resolvers = append(k.resolvers, resolver)
 			}
